#!/bin/sh

set -e

DIRNAME=`dirname "$0"`

. "$DIRNAME/definitions.sh"
. "$DIRNAME/functions.sh.in"

DEFINITIONS_SH="$CCLUB_SCRIPTS/definitions.sh"
if [ $# -ge 2 -a x"$1" = x--definitions-sh ]; then
    test_enable=1
    DEFINITIONS_SH="$2"
    shift 2
fi
FUNCTIONS_SH="$CCLUB_SCRIPTS/functions.sh"
if [ $# -ge 2 -a x"$1" = x--functions-sh ]; then
    test_enable=1
    FUNCTIONS_SH="$2"
    shift 2
fi

if [ $# -lt 2 ]; then
    echo "Usage: `basename "$0"` [--definitions-sh path] [--functions-sh path] config destdir" 1>&2
    exit 1;
fi

DESTDIR="$2"

mkdir -p "$DESTDIR$CCLUB_CONFIG_DCHASH_DIR"
mkdir -p "$DESTDIR$CCLUB_CONFIG_FLAGS_DIR"
mkdir -p "$DESTDIR$CCLUB_CONFIG_GENERATED_DIR"
mkdir -p "$DESTDIR$CCLUB_CONFIG_UCF_DIR"
mkdir -p "$DESTDIR$CCLUB_CONFIG_SHARE"

if [ -n "$test_enable" ]; then
    mkdir -p "$DESTDIR$CCLUB_SCRIPTS"
    cp "$DIRNAME/definitions.sh" "$DESTDIR$CCLUB_SCRIPTS"
    cp "$DIRNAME/functions.sh" "$DESTDIR$CCLUB_SCRIPTS"
fi

exec < "$1"
while IFS=, read action dest src flags; do
    if echo "$action" | grep -q "^#" ||
       [ -z "$action" -a -z "$dest" -a -z "$src" -a -z "$flags" ]; then
        continue;
    fi

    from="$DIRNAME/files/$src"
    to="$DESTDIR$CCLUB_CONFIG_SHARE/$src"
    todir=`dirname "$to"`

    mkdir -p "$todir"

    cp_opts=
    if is_flag_set "$flags" nosymlink; then
        cp_opts="$cp_opts -L"
    else
        cp_opts="$cp_opts -d"
    fi

    if is_flag_set "$flags" glob; then
        frombase=`echo "$from" | sed -e 's/^\(.*\)\..*$/\1/'`
        for match in "$frombase"*; do
            if ! is_flag_set "$flags" nodirglob || [ ! -d "$match" ]; then
                cp ${cp_opts} -r "$match" "$todir"
            fi
        done
    else
        cp ${cp_opts} "$from" "$to"
        if [ -e "${from}.md5sum" ]; then
            cp ${cp_opts} "${from}.md5sum" "${to}.md5sum"
        fi
    fi

    case "$action" in
    script)
        if [ -f "$to" ]; then
            sed -i -e "s%^##definitions.sh##%. ${DEFINITIONS_SH}%" \
                   -e "s%^##functions.sh##%. ${FUNCTIONS_SH}%"     \
                "$to"
        fi
        ;;
    esac
done
